/*
 * 
 * Hardware definitions for B4 testing platform
 *
 * Copyright 2005 STC Metrotek, Russia
 *
 * Based on code:
 *    Copyright 2005 SDG Systems, LLC
 *
 * Based on code:
 *    Copyright 2004 Hewlett-Packard Company.
 *
 * Based on code:
 *    Copyright 2004 Hewlett-Packard Company.
 *
 * Use consistent with the GNU GPL is permitted,
 * provided that this copyright notice is
 * preserved in its entirety in all copies and derived works.
 *
 * History:
 *
 * 2005-10      tony feldman            preliminary version
 */


#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/input.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/mmc/host.h>

#include <asm/mach-types.h>
#include <asm/io.h>
#include <asm/hardware.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>

#include <asm/arch/pxafb.h>
#include <asm/arch/pxa-regs.h>
#include <asm/arch/pxa_keys.h>
#include <asm/arch/ohci.h>
#include <asm/arch/udc.h>
#include <asm/arch/hx4700-gpio.h>
#include <asm/arch/b4-gpio.h>
#include <asm/arch/mmc.h>

#include "../generic.h"
#include "../drivers/serial/pxa-serial.h"
/* SD/MMC controller */

static struct pxamci_platform_data b4_mci_platform_data;
#define B4_MMC_DETECT_GPIO      105
#define B4_MMC_WP_GPIO          102
#define B4_MMC_PWR_GPIO         93
static int b4_mci_init(struct device *dev, irqreturn_t (*b4_carddetect_int)(int,                                void *, struct pt_regs *), void *data)
{
  int err = 0;
  /* setup SD/MMC gpio */
  pxa_gpio_mode( GPIO32_MMCCLK | GPIO_ALT_FN_2_OUT);
  pxa_gpio_mode( GPIO92_MMCDAT0 | GPIO_ALT_FN_1_IN);
  pxa_gpio_mode( GPIO109_MMCDAT1 | GPIO_ALT_FN_1_IN);
  pxa_gpio_mode( GPIO110_MMCDAT2 | GPIO_ALT_FN_1_IN);
  pxa_gpio_mode( GPIO111_MMCDAT3 | GPIO_ALT_FN_1_IN);
  pxa_gpio_mode( GPIO112_MMCCMD | GPIO_ALT_FN_1_IN);
  pxa_gpio_mode( B4_MMC_DETECT_GPIO | GPIO_IN);  
  pxa_gpio_mode( B4_MMC_WP_GPIO | GPIO_IN);
  pxa_gpio_mode( B4_MMC_PWR_GPIO  | GPIO_OUT);
 
  b4_mci_platform_data.detect_delay = msecs_to_jiffies(250);

  set_irq_type(IRQ_GPIO(B4_MMC_DETECT_GPIO), IRQT_BOTHEDGE);

  err = request_irq(IRQ_GPIO(B4_MMC_DETECT_GPIO), b4_carddetect_int, 
                    SA_INTERRUPT, "SD/MMC CD", data);
  if (err) {
    printk(KERN_ERR "b4_mci_init: MMC/SD: can't request MMC card detect IRQ\n");
  }

  return err;
}

static int b4_mci_get_ro(struct device *dev)
{
        if ((GPLR(B4_MMC_WP_GPIO) & GPIO_bit(B4_MMC_WP_GPIO)) == 0) {
                printk("Card is write-protected\n");
                return 1;
        }
        return 0;
}

static void b4_mci_set_power(struct device* dev, unsigned int vdd) {
        struct pxamci_platform_data *pdata = dev->platform_data;
        
        if ((1 < vdd) & pdata->ocr_mask) {
                GPSR(B4_MMC_PWR_GPIO) = ~GPIO_bit(B4_MMC_PWR_GPIO);
        } else {
                GPSR(B4_MMC_PWR_GPIO) = GPIO_bit(B4_MMC_PWR_GPIO);
        }

}

static void b4_mci_exit(struct device *dev, void *data)
{
        free_irq(IRQ_GPIO(B4_MMC_DETECT_GPIO), data);
}

static struct pxamci_platform_data b4_mci_platform_data = {
        .ocr_mask       = MMC_VDD_32_33|MMC_VDD_33_34,
        .init           = b4_mci_init,
        .get_ro         = b4_mci_get_ro,
        .setpower       = b4_mci_set_power,
        .exit           = b4_mci_exit,
};

/* [yl] */

#define REV1_ALARM_LED_GPIO (11)
#define REV2_ALARM_LED_GPIO (114)

#define REV1_PWR_LED_GPIO   (12)
#define REV2_PWR_LED_GPIO   REV1_PWR_LED_GPIO


void pxa_init_leds_gpio( )
{
   if( system_rev == 1 ){
     pxa_gpio_mode( REV1_ALARM_LED_GPIO | GPIO_OUT | GPIO_DFLT_LOW ); 
     pxa_gpio_mode( REV1_PWR_LED_GPIO | GPIO_OUT | GPIO_DFLT_HIGH );
   } else if( system_rev == 2){
     pxa_gpio_mode( REV2_ALARM_LED_GPIO | GPIO_OUT | GPIO_DFLT_LOW );
     pxa_gpio_mode( REV2_PWR_LED_GPIO | GPIO_OUT | GPIO_DFLT_HIGH );
   }

}

/* USB Device Controller */
struct b4_core_funcs {
            int (*udc_detect)( void );
};

static struct b4_core_funcs core_funcs;

static int
udc_detect(void)
{
	if (core_funcs.udc_detect != NULL)
		return core_funcs.udc_detect();
	else
		return 0;
}

static void
udc_enable(int cmd) 
{
	switch (cmd)
	{
		case PXA2XX_UDC_CMD_DISCONNECT:
			printk (KERN_NOTICE "USB cmd disconnect\n");
			UP2OCR &= ~UP2OCR_DPPUE;
			break;

		case PXA2XX_UDC_CMD_CONNECT:
			printk (KERN_NOTICE "USB cmd connect\n");
			UP2OCR |= UP2OCR_DPPUE;
                        break;
	}
}

static struct pxa2xx_udc_mach_info b4_udc_mach_info = {
	.udc_is_connected = udc_detect,
	.udc_command      = udc_enable,
};


//[artem]
/*
 * USB Host (OHCI)
 */
static int b4_ohci_init( struct device *dev )
{

  /* setup Port1 GPIO pin. */
  pxa_gpio_mode( GPIO88_USBHPWR1 | GPIO_ALT_FN_1_IN);   
  pxa_gpio_mode( GPIO89_USBHPEN1 | GPIO_ALT_FN_2_OUT);  

  //UHCHR_SSEPx Sleep Standby Enable for Port x 0-enable
  //UHCHR_PCPL- Polarity of the USBHPEN =0 output signals is active high.
  //UHCHR_PSPL- Polarity of the USBHPWR =0 input signals is active high.
  UHCHR = (UHCHR | UHCHR_PSPL) &  ~(UHCHR_SSEP1 | UHCHR_PCPL );
  
  /*  No over-current protection supported. */
  UHCRHDA |= UHCRHDA_NOCP;

  return 0;
}

static struct pxaohci_platform_data b4_ohci_platform_data = {
        .port_mode      = PMM_PERPORT_MODE,
	.init		= b4_ohci_init,
};



/* PXA2xx Keys */

static struct pxa_keys_button b4_button_table[] = {
  { KEY_SUSPEND, 0, 1, 0 },
};

static struct pxa_keys_platform_data b4_pxa_keys_data = {
  .buttons = b4_button_table,
  .nbuttons = ARRAY_SIZE(b4_button_table),
};

static struct platform_device b4_pxa_keys = {
  .name = "pxa2xx-keys",
  .dev = {
    .platform_data = &b4_pxa_keys_data,
  },
};

static struct platform_device b4_ts = {
  .name = "ts",
  .dev = {
    .platform_data = &b4_ts,
  },
};

static struct platform_device b4_kbd = {
  .name = "kbd",
  .dev = {
    .platform_data = &b4_kbd,
  },
};

static struct map_desc b4_io_desc[] = {
          {0xf5000000, __phys_to_pfn(PXA_CS1_PHYS), 0x00020000, MT_DEVICE},
          {0xf5100000, __phys_to_pfn(PXA_CS3_PHYS), (1 << 16), MT_DEVICE},
};

static void __init b4_map_io(void)
{
  pxa_map_io();

  iotable_init(b4_io_desc, ARRAY_SIZE(b4_io_desc));
  
  //Power Manager Wake-Up Enable Register
  PWER = PWER_GPIO0 | PWER_RTC;   
  
  //Power Manager Rising-Edge Detect Enable Register
  PFER = PWER_GPIO0 | PWER_RTC; 
  
  //Power Manager Falling-Edge Detect Enable Register
  PRER = 0;
  
  //Power Manager General Configuration Register
  PCFR = PCFR_OPDE | PCFR_GPR_EN;
}

static void __init b4_init_irq(void)
{
  pxa_init_irq();
}

/* Core Hardware Functions */

static struct pxafb_mach_info prime_view_70wx2 __initdata= {
        .pixclock     = 25000,     // (160756 > 180849)
        .bpp          = 16,         // BPP (0x4 == 16bits)
        .xres         = 800,        // PPL + 1
        .yres         = 480,        // LPP + 1
        .hsync_len    = 64,          // HSW + 1
        .vsync_len    = 2,          // VSW + 1
        .left_margin  = 86,          // BLW + 1
        .upper_margin = 33,          // BFW + 1
        .right_margin = 42,          // ELW + 1
        .lower_margin = 33,          // EFW + 1
        .sync         = 0,
        .lccr0        = LCCR0_Act, // | LCCR0_Sngl | LCCR0_Color,
        .lccr3        = 0,//0,//LCCR3_V | LCCR3_PixFlEdg | LCCR3_Acb(0),

        //.pxafb_backlight_power = ,
        //.pxafb_lcd_power =       ,
};



static void __init b4_init( void )
{
  set_pxa_fb_info(&prime_view_70wx2);
  
  
  
  pxa_gpio_mode( B4_GPIO_FF_TXD );
  pxa_gpio_mode( B4_GPIO_FF_RTS );
  pxa_gpio_mode( B4_GPIO_FF_RXD );
  pxa_gpio_mode( B4_GPIO_FF_CTS );
  pxa_gpio_mode( B4_GPIO_FF_DCD );
  pxa_gpio_mode( B4_GPIO_FF_DSR );
  pxa_gpio_mode( B4_GPIO_FF_RI );
  pxa_gpio_mode( B4_GPIO_FF_DTR );
  
  platform_device_register(&b4_pxa_keys);
  platform_device_register(&b4_ts);
  platform_device_register(&b4_kbd);

  /* SD/MMC [yl] */
  pxa_set_mci_info(&b4_mci_platform_data);

  pxa_init_leds_gpio();
  /* [yl] */ 
  pxa_set_udc_info( &b4_udc_mach_info );
  /*  USB Host (OHCI)    [artem] */
  pxa_set_ohci_info( &b4_ohci_platform_data );
}


MACHINE_START(B4, "b4")
        /* Maintainer STC Metrotek, Russia */
//	.phys_ram	= 0xa0000000, /* commented since 2.6.17-hh2 */
	.phys_io	= 0x40000000,
	.io_pg_offst	= (io_p2v(0x40000000) >> 18) & 0xfffc,
        .boot_params	= 0xa0000100,
        .map_io		= b4_map_io,
        .init_irq	= b4_init_irq,
        .timer          = &pxa_timer,
        .init_machine   = b4_init,
MACHINE_END

/* vim600: set sw=8 expandtab : */
