/*
 * LED interface for Himalaya, the HTC PocketPC.
 *
 * License: GPL
 *
 * Author: Luke Kenneth Casson Leighton, Copyright(C) 2004
 *
 * Copyright(C) 2004, Luke Kenneth Casson Leighton.
 *
 * History:
 *
 * 2004-02-19	Luke Kenneth Casson Leighton	created.
 *
 */
 
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/leds.h>
#include <linux/soc/asic3_base.h>

#include <asm/io.h>
#include <asm/hardware/ipaq-asic3.h>
#include <asm/hardware/ipaq-ops.h>
#include <asm/arch/htcblueangel-asic.h>
#include <asm/mach-types.h>
#include <asm/arch/ipaq.h>

//#include "blueangel_leds.h"

#ifdef DEBUG
#define dprintk(x...) printk(x)
#else
#define dprintk(x...)
#endif


extern struct platform_device blueangel_asic3;
extern struct ipaq_model_ops ipaq_model_ops;
static volatile void *virt;


#define CLOCK_CDEX_LED0      (1 << 6)  /* clock for led 0? */
#define CLOCK_CDEX_LED1      (1 << 7)  /* clock for led 1? */

struct blueangel_led_data {
	int                     hw_num;                 // 0 (amber), 1 (green), 2 (blue)
	int                     duty_time;
	int                     cycle_time;
	int                     registered;
	int                     brightness;
	int                     color;
	struct led_properties   props;
};

#define to_blueangel_led_data(d) container_of(d, struct blueangel_led_data, props)

static int
led_read(int reg)
{
	volatile unsigned short *addr=((volatile unsigned short *)(virt+0xa));	
	volatile unsigned char *data=((volatile unsigned char *)(virt+0xc));	

	*addr |= 0x80;
	*addr = (*addr & 0xff80) | (reg & 0x7f);

	dprintk("led_read[0x%x]=0x%x\n", reg, *data);	
	return *data;
}

static void
led_write(int reg, int val)
{
	volatile unsigned short *addr=((volatile unsigned short *)(virt+0xa));	
	volatile unsigned short *data=((volatile unsigned short *)(virt+0xc));	

	dprintk("led_write[0x%x]=0x%x\n", reg, val);	
	*addr = (*addr & 0xff80) | (reg & 0x7f);
	*addr &= 0xff7f;

	*data = (*data & 0xff00) | (val & 0xff);

}

void blueangel_set_led (int led, int brightness, int duty_time, int cycle_time)
{
	dprintk("blueangel_set_led\n");
	if (led <= 2) {
		if (brightness) {
			duty_time=(duty_time*128+500)/1000;
			cycle_time=(cycle_time*128+500)/1000;
		} else {
			duty_time=0;
			cycle_time=1;
		}
	}
	if (led == 6 || led == 7) {
		if (brightness) {
			duty_time=(duty_time*16+500)/1000;
			cycle_time=(cycle_time*16+500)/1000;
		} else {
			duty_time=0;
			cycle_time=1;
		}
	}
	switch(led) {
	case 0:
		ipaq_asic3_set_led(&blueangel_asic3.dev, 0, duty_time, cycle_time);
		break;
	case 1:
		ipaq_asic3_set_led(&blueangel_asic3.dev, 1, duty_time, cycle_time);
		break;
	case 2:
		ipaq_asic3_set_led(&blueangel_asic3.dev, 0, duty_time, cycle_time);
		ipaq_asic3_set_led(&blueangel_asic3.dev, 1, duty_time, cycle_time);
		break;
	case 6:
		led_write(0, duty_time);
		led_write(1, cycle_time-duty_time);
		if (brightness) {
			led_write(4, led_read(4) | 0x1);
			led_write(0x20, led_read(0x20) & ~0x4);
		} else {
			led_write(4, led_read(4) &  ~0x1);
			led_write(0x20, led_read(0x20) | 0x4);
		}
		break;
	case 7:
		led_write(2, duty_time);
		led_write(3, cycle_time-duty_time);
		if (brightness) {
			led_write(4, led_read(4) | 0x2);
			led_write(0x20, led_read(0x20) & ~0x8);
		} else {
			led_write(4, led_read(4) &  ~0x2);
			led_write(0x20, led_read(0x20) | 0x8);
		}
		break;
	}
}

EXPORT_SYMBOL(blueangel_set_led);


int blueangel_led_brightness_get(struct device *dev, struct led_properties *props)
{
	struct blueangel_led_data *data = to_blueangel_led_data(props);

	return data->brightness;
}

void blueangel_led_brightness_set(struct device *dev, struct led_properties *props, int value)

/* Brightness levels supported by the hardware: 0 (off) and 100 (on)*/

{
	struct blueangel_led_data *data = to_blueangel_led_data(props);

	data->brightness = value > 0 ? 100 : 0;
	if (! data->brightness && (data->hw_num < 2 || data->hw_num == 6 || data->hw_num ==7 )) {
		data->duty_time=1000;
		data->cycle_time=1000;
	}
	switch (data->hw_num) {
	case 0:
	case 1:
	case 2:
	case 6:
	case 7:
		blueangel_set_led(data->hw_num, data->brightness, data->duty_time, data->cycle_time);
		break;
	case 3:
		ipaq_asic3_set_gpio_out_b(&blueangel_asic3.dev, GPIOB_PHONEL_PWR_ON, (value > 0) ? GPIOB_PHONEL_PWR_ON : 0);
		break;
	case 4:
		ipaq_asic3_set_gpio_out_c(&blueangel_asic3.dev, GPIOC_KEYBL_PWR_ON, (value > 0) ? GPIOC_KEYBL_PWR_ON : 0);
		break;
	case 5:
		ipaq_asic3_set_gpio_out_b(&blueangel_asic3.dev, GPIOB_VIBRA_PWR_ON, (value > 0) ? GPIOB_VIBRA_PWR_ON : 0);
		break;
	}
}

int
blueangel_led_cycle_get(struct device *dev, struct led_properties *props)
{
	struct blueangel_led_data *data = to_blueangel_led_data(props);

	return data->cycle_time;
}

void
blueangel_led_cycle_set(struct device *dev, struct led_properties *props, int value)
{
	struct blueangel_led_data *data = to_blueangel_led_data(props);

	data->cycle_time=value;
	if (data->brightness)
		blueangel_set_led(data->hw_num, data->brightness, data->duty_time, data->cycle_time);
		
}

int
blueangel_led_duty_get(struct device *dev, struct led_properties *props)
{
	struct blueangel_led_data *data = to_blueangel_led_data(props);

	return data->duty_time;
}

void
blueangel_led_duty_set(struct device *dev, struct led_properties *props, int value)
{
	struct blueangel_led_data *data = to_blueangel_led_data(props);

	data->duty_time=value;
	if (data->brightness)
		blueangel_set_led(data->hw_num, data->brightness, data->duty_time, data->cycle_time);
}

static struct blueangel_led_data leds[] = {
      	{ 
		.hw_num=0,
		.duty_time=1000,
		.cycle_time=1000,
		.props  = {
			.owner	       = THIS_MODULE,
			.name	       = "red",
			.color	       = "red",
			.brightness_get = blueangel_led_brightness_get,
			.brightness_set = blueangel_led_brightness_set,
//			.hw_cycle_get   = blueangel_led_cycle_get,
//			.hw_cycle_set   = blueangel_led_cycle_set,
			.hw_duty_get    = blueangel_led_duty_get,
			.hw_duty_set    = blueangel_led_duty_set,
		}
	},
	{
		.hw_num=1,
		.duty_time=1000,
		.cycle_time=1000,
		.props  = {
			.owner	       = THIS_MODULE,
			.name	       = "green",
			.color	       = "green",
			.brightness_get = blueangel_led_brightness_get,
			.brightness_set = blueangel_led_brightness_set,
			.hw_cycle_get   = blueangel_led_cycle_get,
			.hw_cycle_set   = blueangel_led_cycle_set,
			.hw_duty_get    = blueangel_led_duty_get,
			.hw_duty_set    = blueangel_led_duty_set,
		}
	},
	{
		.hw_num=2,
		.duty_time=1000,
		.cycle_time=1000,
		.props  = {
			.owner	       = THIS_MODULE,
			.name	       = "yellow",
			.color	       = "yellow",
			.brightness_get = blueangel_led_brightness_get,
			.brightness_set = blueangel_led_brightness_set,
			.hw_cycle_get   = blueangel_led_cycle_get,
			.hw_cycle_set   = blueangel_led_cycle_set,
			.hw_duty_get    = blueangel_led_duty_get,
			.hw_duty_set    = blueangel_led_duty_set,
		}
	},
	{
		.hw_num=3,
		.brightness=100,
		.props  = {
			.owner	       = THIS_MODULE,
			.name	       = "phone",
			.brightness_get = blueangel_led_brightness_get,
			.brightness_set = blueangel_led_brightness_set,
		}
	},
	{
		.hw_num=4,
		.props  = {
			.owner	       = THIS_MODULE,
			.name	       = "keyboard",
			.color	       = "blue",
			.brightness_get = blueangel_led_brightness_get,
			.brightness_set = blueangel_led_brightness_set,
		}
	},
	{
		.hw_num=5,
		.props  = {
			.owner	       = THIS_MODULE,
			.name	       = "vibra",
			.brightness_get = blueangel_led_brightness_get,
			.brightness_set = blueangel_led_brightness_set,
		}
	},
	{
		.hw_num=6,
		.duty_time=1000,
		.cycle_time=1000,
		.props  = {
			.owner	       = THIS_MODULE,
			.name	       = "blue",
			.color	       = "blue",
			.brightness_get = blueangel_led_brightness_get,
			.brightness_set = blueangel_led_brightness_set,
			.hw_cycle_get   = blueangel_led_cycle_get,
			.hw_cycle_set   = blueangel_led_cycle_set,
			.hw_duty_get    = blueangel_led_duty_get,
			.hw_duty_set    = blueangel_led_duty_set,
		}
	},
	{
		.hw_num=7,
		.duty_time=1000,
		.cycle_time=1000,
		.props  = {
			.owner	       = THIS_MODULE,
			.name	       = "left_green",
			.color	       = "green",
			.brightness_get = blueangel_led_brightness_get,
			.brightness_set = blueangel_led_brightness_set,
			.hw_cycle_get   = blueangel_led_cycle_get,
			.hw_cycle_set   = blueangel_led_cycle_set,
			.hw_duty_get    = blueangel_led_duty_get,
			.hw_duty_set    = blueangel_led_duty_set,
		}
	},
};

static int blueangel_led_probe(struct device *dev)
{
	int i,ret=0;

	dprintk("blueangel_led_probe\n");
        /* Turn on the LED controllers in CDEX */
        asic3_set_clock_cdex(&blueangel_asic3.dev,
                CLOCK_CDEX_LED0 | CLOCK_CDEX_LED1 | CLOCK_CDEX_LED2,
                CLOCK_CDEX_LED0 | CLOCK_CDEX_LED1 | CLOCK_CDEX_LED2
        );
#if 0
	ipaq_model_ops.set_led = blueangel_set_led;
#endif
	blueangel_set_led(RED_LED, 0, 0, 0);
	blueangel_set_led(GREEN_LED, 0, 0, 0);
	blueangel_set_led(YELLOW_LED, 0, 0, 0);
	blueangel_set_led(6, 0, 0, 0);
	blueangel_set_led(7, 0, 0, 0);
	for (i = 0; i < ARRAY_SIZE(leds); i++) {
	       ret = leds_device_register(dev, &leds[i].props);
	       leds[i].registered = 1;
	       if (unlikely(ret)) {
		       printk(KERN_WARNING "Unable to register blueangel led %s\n", leds[i].props.color);
		       leds[i].registered = 0;
	       }
	}
	return ret;
}

static int blueangel_led_remove(struct device *dev)
{
	int i;

	dprintk("blueangel_led_remove\n");

	ipaq_model_ops.set_led = NULL;

	for (i = 0; i < ARRAY_SIZE(leds); i++) {
	       if (leds[i].registered) {
		       leds_device_unregister(&leds[i].props);
	       }
	}
	return 0;
}

static int 
blueangel_led_suspend(struct device *dev, u32 state, u32 level)
{
//	if (level == SUSPEND_POWER_DOWN) {
		ipaq_asic3_set_gpio_out_b(&blueangel_asic3.dev, GPIOB_PHONEL_PWR_ON, 0);
		ipaq_asic3_set_gpio_out_c(&blueangel_asic3.dev, GPIOC_KEYBL_PWR_ON, 0);
		ipaq_asic3_set_gpio_out_b(&blueangel_asic3.dev, GPIOB_VIBRA_PWR_ON, 0);
//	}
	return 0;
}

static int
blueangel_led_resume(struct device *dev, u32 level)
{
//	if (level == RESUME_POWER_ON) {
		dprintk("blueangel_led_resume brightness=%d\n", leds[3].brightness);
		ipaq_asic3_set_gpio_out_b(&blueangel_asic3.dev, GPIOB_PHONEL_PWR_ON, leds[3].brightness ? GPIOB_PHONEL_PWR_ON : 0);
		ipaq_asic3_set_gpio_out_c(&blueangel_asic3.dev, GPIOC_KEYBL_PWR_ON, leds[4].brightness ? GPIOC_KEYBL_PWR_ON : 0);
		ipaq_asic3_set_gpio_out_b(&blueangel_asic3.dev, GPIOB_VIBRA_PWR_ON, leds[5].brightness ? GPIOB_VIBRA_PWR_ON : 0);
//	}
	
	return 0;
}


static struct device_driver blueangel_led_driver = {
	.name   = "blueangel_led",
	.probe  = blueangel_led_probe,
	.remove = blueangel_led_remove,
	.suspend = blueangel_led_suspend,
	.resume = blueangel_led_resume,
	.bus    = &platform_bus_type,

};

static struct platform_device blueangel_led_dev = {
	.name 	= "blueangel_led",
};


static int blueangel_led_init (void)
{
	int ret = 0;

	if (! machine_is_blueangel() )
		return -ENODEV;

	virt=ioremap(0x11000000, 4096);
	dprintk("0x%x 0x%x 0x%x 0x%x 0x%x\n", led_read(0), led_read(1), led_read(2), led_read(3), led_read(4));
	dprintk("0x%x 0x%x 0x%x\n", led_read(0x20), led_read(0x21), led_read(0x22));
	ret=driver_register(&blueangel_led_driver);
	if (! ret) 
		ret=platform_device_register(&blueangel_led_dev);
	return ret;
}

static void blueangel_led_exit (void)
{
	iounmap(virt);
	platform_device_unregister(&blueangel_led_dev);
	driver_unregister(&blueangel_led_driver);
}

module_init (blueangel_led_init);
module_exit (blueangel_led_exit);

MODULE_LICENSE("GPL");
