/*
 * linux/arch/arm/mach-pxa/htcsable/htcsable.c
 *
 *  Support for the Intel XScale based Palm PDAs. Only the LifeDrive is
 *  supported at the moment.
 *
 *  Author: Alex Osborne <bobofdoom@gmail.com>
 *
 *  USB stubs based on aximx30.c (Michael Opdenacker)
 *
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/fb.h>
#include <linux/platform_device.h>

#include <asm/mach-types.h>
#include <asm/mach/arch.h>

#include <asm/arch/hardware.h>
#include <asm/arch/pxafb.h>
#include <asm/arch/pxa-regs.h>
#include <asm/arch/udc.h>
#include <asm/arch/serial.h>

#include <asm/arch/htcsable-gpio.h>
#include <asm/arch/htcsable-asic.h>

#include <asm/hardware/ipaq-asic3.h>
#include <linux/soc/asic3_base.h>

#include "../generic.h"

#include "htcsable_bt.h"

/*
 *  * Bluetooth - Relies on other loadable modules, like ASIC3 and Core,
 *   * so make the calls indirectly through pointers. Requires that the
 *    * hx4700 bluetooth module be loaded before any attempt to use
 *     * bluetooth (obviously).
 *      */

static struct htcsable_bt_funcs bt_funcs;

static void
htcsable_bt_configure( int state )
{
  if (bt_funcs.configure != NULL)
    bt_funcs.configure( state );
}


static struct pxafb_mach_info htcsable_lcd __initdata = {
	.pixclock		= 480769, // LCCR4 bit is set!
	.xres			= 240,
	.yres			= 240,
	.bpp			= 16,
	.hsync_len		= 4,
	.vsync_len		= 2,
	.left_margin		= 12,
	.right_margin		= 8,
	.upper_margin		= 3,
	.lower_margin		= 3,

//	.sync			= FB_SYNC_HOR_LOW_ACT|FB_SYNC_VERT_LOW_ACT,

	/* fixme: this is a hack, use constants instead. */
	/*.lccr0			= 0x05b008b9,
	.lccr3			= 0x04700019,*/
	.lccr0			= 0x042000b1,
	.lccr3			= 0x04700019,

};

static struct platform_device htcsable_udc       = { .name = "htcsable_udc", };
static struct platform_device htcsable_ts        = { .name = "htcsable_ts", };

/* Bluetooth */

static struct platform_device htcsable_bt = {
  .name = "htcsable_bt",
  .id = -1,
  .dev = {
    .platform_data = &bt_funcs,
  },
};

static struct platform_device *htcsable_asic3_devices[] __initdata = {
//	&htcsable_lcd,
	&htcsable_ts,
	&htcsable_bt,
	&htcsable_udc,
};

static struct asic3_platform_data htcsable_asic3_platform_data = {

   /*
    * These registers are configured as they are on Wince.
    */
        .gpio_a = {
		.dir            = 0xbffb,
		.init           = 0xc087,
		.sleep_out      = 0x0000,
		.batt_fault_out = 0x0000,
		.alt_function   = 0x6000, //
		.sleep_conf     = 0x000c,
        },
        .gpio_b = {
		.dir            = 0xe008,
		.init           = 0x1762,
		.sleep_out      = 0x0000,
		.batt_fault_out = 0x0000,
		.alt_function   = 0x0000, //
                .sleep_conf     = 0x000c,
        },
        .gpio_c = {
                .dir            = 0xfff7,
                .init           = 0x8640,
                .sleep_out      = 0x0000,
                .batt_fault_out = 0x0000,
		.alt_function   = 0x0038, // GPIOC_LED_RED | GPIOC_LED_GREEN | GPIOC_LED_BLUE
                .sleep_conf     = 0x000c,
        },
        .gpio_d = {
		.dir            = 0xffff,
		.init           = 0xd000,
		.sleep_out      = 0x0000,
		.batt_fault_out = 0x0000,
		.alt_function   = 0x0000, //
		.sleep_conf     = 0x0008,
        },
	.bus_shift = 1,

	.child_platform_devs     = htcsable_asic3_devices,
	.num_child_platform_devs = ARRAY_SIZE(htcsable_asic3_devices),
};

static struct resource htcsable_asic3_resources[] = {
	[0] = {
		.start	= HTCSABLE_ASIC3_GPIO_PHYS,
		.end	= HTCSABLE_ASIC3_GPIO_PHYS + IPAQ_ASIC3_MAP_SIZE,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= HTCSABLE_IRQ(ASIC3_EXT_INT),
		.end	= HTCSABLE_IRQ(ASIC3_EXT_INT),
		.flags	= IORESOURCE_IRQ,
	},
	[2] = {
		.start  = HTCSABLE_ASIC3_MMC_PHYS,
		.end    = HTCSABLE_ASIC3_MMC_PHYS + IPAQ_ASIC3_MAP_SIZE,
		.flags  = IORESOURCE_MEM,
	},
	[3] = {
		.start  = HTCSABLE_IRQ(ASIC3_SDIO_INT_N),
		.flags  = IORESOURCE_IRQ,
	},
};

struct platform_device htcsable_asic3 = {
	.name           = "asic3",
	.id             = 0,
	.num_resources  = ARRAY_SIZE(htcsable_asic3_resources),
	.resource       = htcsable_asic3_resources,
	.dev = { .platform_data = &htcsable_asic3_platform_data, },
};
EXPORT_SYMBOL(htcsable_asic3);

static struct platform_device *devices[] __initdata = {
	&htcsable_asic3,
};

#if 0
/****************************************************************
 * USB client controller
 ****************************************************************/

static void udc_command(int cmd)
{
	switch (cmd)
	{
		case PXA2XX_UDC_CMD_DISCONNECT:
			printk(KERN_NOTICE "USB cmd disconnect\n");
//                        GPSR_BIT(GPIO_NR_HTCAPACHE_USB);
			break;
		case PXA2XX_UDC_CMD_CONNECT:
			printk(KERN_NOTICE "USB cmd connect\n");
//                        GPCR_BIT(GPIO_NR_HTCAPACHE_USB);
			break;
	}
}

static struct pxa2xx_udc_mach_info htcsable_udc_mach_info = {
	.udc_command      = udc_command,
};
#endif

static struct platform_pxa_serial_funcs htcsable_pxa_bt_funcs = {
  .configure = htcsable_bt_configure,
};

static void __init htcsable_map_io(void)
{
  pxa_map_io();

  ffuart_device.dev.platform_data = &htcsable_pxa_bt_funcs;

  printk("CKEN=0x%x CKEN11_USB=0x%x\n", CKEN, CKEN11_USB);
  pxa_set_cken(CKEN11_USB, 1);
  printk("CKEN=0x%x\n", CKEN);
}


static void __init htcsable_init(void)
{
	set_pxa_fb_info( &htcsable_lcd );

	platform_device_register(&htcsable_asic3);
//	platform_add_devices( devices, ARRAY_SIZE(devices) );
//	pxa_set_udc_info( &htcsable_udc_mach_info );
}

MACHINE_START(HW6900, "HTC Sable")
	.phys_io	= 0x40000000,
	.io_pg_offst    = (io_p2v(0x40000000) >> 18) & 0xfffc,
	.boot_params	= 0xa0000100,
	.map_io 	= htcsable_map_io,
	.init_irq	= pxa_init_irq,
	.timer  	= &pxa_timer,
	.init_machine	= htcsable_init,
MACHINE_END

