/*
 * linux/arch/arm/common/tc6393xb_soc.c
 *
 * Toshiba TC6393 support
 * Copyright (c) 2005 Dirk Opfer
 * Copyright (c) 2005 Ian Molton <spyro@f2s.com>
 *
 *	Based on code written by Sharp/Lineo for 2.4 kernels
 *	Based on locomo.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This file contains all generic TC6393 support.
 *
 * 19/03/2005 IM Fixed IRQ code
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/device.h>
#include <linux/slab.h>
#include <linux/spinlock.h>
#include <linux/soc-device.h>
#include <linux/soc/tmio_mmc.h>
#include <linux/soc/tmio_nand.h>
#include <linux/soc/tmio_ohci.h>

#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mach/irq.h>
#include <asm/arch/pxa-regs.h>

#include <asm/hardware/tc6393.h>

struct tc6393xb_data
{
	int irq_base, irq_nr;
	unsigned long mapbase;
	struct platform_device *devices;
	int ndevices;
};

struct tc6393xb_block
{
	platform_device_id id;
	char *name;
	unsigned long ctl_start, ctl_end;
	unsigned long cnf_start, cnf_end;
	int irq;
	void *hwconfig;
};

/* Setup the TC6393XB NAND flash controllers configuration registers */
static void tc6393xb_nand_hwinit(struct platform_device *sdev) {
	
	struct tc6393xb_data *chip = (struct tc6393xb_data *)dev_get_drvdata(sdev->dev.parent);
	
	/* Sequence: 
	 * SMD Buffer ON (gpio related)
	 * Enable the clock (SCRUNEN)
	 * Set the ctl reg base address
	 * Enable the ctl reg
	 * Configure power control (control bt PCNT[1,0] 4ms startup delay)
	 */
	printk("nand_hwinit(0x%08lx)\n", chip->mapbase);
	/* (89h) SMD Buffer ON By TC6393XB SystemConfig gpibfc1*/
	writeb(0xff, chip->mapbase + TC6393_SYS_GPIBCR1);

//	return 0;
}

static void tc6393xb_nand_suspend(struct platform_device *sdev) {
	printk("nand_suspend()\n");
//	return 0;
}

static void tc6393xb_nand_resume(struct platform_device *sdev) {
	printk("nand_resume()\n");
//	return 0;
}

static struct tmio_nand_hwconfig tc6393xb_nand_hwconfig = {
	.hwinit  = tc6393xb_nand_hwinit,
	.suspend = tc6393xb_nand_suspend,
	.resume  = tc6393xb_nand_resume,
};

static void tc6393xb_mmc_set_clock(struct platform_device *sdev, int state) {
	struct tc6393xb_data *chip = (struct tc6393xb_data *)dev_get_drvdata(sdev->dev.parent);
	unsigned char tmp;

	printk("mmc_set_clock(0x%08lx)\n", chip->mapbase);
	if(state == MMC_CLOCK_ENABLED){
		tmp = readb(chip->mapbase + 0x98);
		writeb(tmp | CK32KEN, chip->mapbase + 0x98);
	//FIXME - need to disable too...
	}
}

static struct tmio_mmc_hwconfig tc6393xb_mmc_hwconfig = {
        .set_mmc_clock  = tc6393xb_mmc_set_clock,
};

static void tc6393xb_ohci_hwinit(struct platform_device *sdev) {
	struct tc6393xb_data *chip = (struct tc6393xb_data *)dev_get_drvdata(sdev->dev.parent);
	unsigned char tmp;

	// enable usb clock - sysconf + 0x98, halfword, bit D1 (0x2)
	// FIXME - setting this reg could use some locking...
	tmp = readb(chip->mapbase + 0x98);
	writeb(tmp | USBCLK, chip->mapbase + 0x98);
	// enable usb function - sysconf + 0xe0  bit D0 (0x1)
	writeb(0x1, chip->mapbase + 0xe0);
}

static struct tmio_ohci_hwconfig tc6393xb_ohci_hwconfig = {
        .hwinit  = tc6393xb_ohci_hwinit,
};


static struct tc6393xb_block tc6393xb_blocks[] = {
	{
		.name      = "tmio_nand",
		.id        = { TMIO_NAND_DEVICE_ID },
		.cnf_start = TC6393XB_NAND_CNF_BASE,
		.cnf_end   = TC6393XB_NAND_CNF_BASE + 0xff,
		.ctl_start = TC6393XB_NAND_CTL_BASE,
		.ctl_end   = TC6393XB_NAND_CTL_BASE + 0xff,
		.irq       = -1,
		.hwconfig  = &tc6393xb_nand_hwconfig,
	},
/*	{
		.id    = TMIO_FB_DEVICE_ID,
		.name  = "tmio_fb",
		.ctl_start = TC6393_GC_CTL_BASE,
		.ctl_end   = TC6393_GC_CTL_BASE + 0x2ff,
		.irq   = -1,
	},
*/
	{
		.name  = "tmio_mmc",
		.id    = { TMIO_MMC_DEVICE_ID },
		.cnf_start = TC6393_MMC_CONF_BASE,
		.cnf_end   = TC6393_MMC_CONF_BASE + 0xff,
		.ctl_start = TC6393_MMC_CTL_BASE,
		.ctl_end   = TC6393_MMC_CTL_BASE + 0x1ff,
		.irq       = TC6393_MMC_IRQ,
		.hwconfig  = &tc6393xb_mmc_hwconfig,
	},
	{
		.id    = { TMIO_USB_DEVICE_ID },
		.name  = "tmio_ohci",
		.cnf_start = TC6393XB_USB_CNF_BASE,
		.cnf_end   = TC6393XB_USB_CNF_BASE + 0xff,
		.ctl_start = TC6393XB_USB_CTL_BASE,
		.ctl_end   = TC6393XB_USB_CTL_BASE + 0xff,
		.irq   = TC6393XB_USB_IRQ,
		.hwconfig  = &tc6393xb_ohci_hwconfig,
	},
};




/** TC6393 interrupt handling stuff.
 * NOTE: TC6393 has a 1 to many mapping on all of its IRQs.
 * that is, there is only one real hardware interrupt
 * we determine which interrupt it is by reading some IO memory.
 * We have two levels of expansion, first in the handler for the
 * hardware interrupt we generate an interrupt
 * IRQ_TC6393_*_BASE and those handlers generate more interrupts
 *
 */
static void tc6393_irq_handler(unsigned int irq, struct irqdesc *desc,
			struct pt_regs *regs)
{
	int req, i;
	struct irqdesc *d = NULL;
	struct tc6393xb_data *tc6393xb;
	
	tc6393xb = desc->data;
	
	/* Acknowledge the parent IRQ */
	desc->chip->ack(irq);
  
	while ( (req = (readb(tc6393xb->mapbase + TC6393_SYS_ISR)
			& ~(readb(tc6393xb->mapbase + TC6393_SYS_IMR)))) ) {
		for (i = 0; i <= 7; i++) {
			if ((req & (1<<i))) {
				if(i != 1)  printk("IRQ! from %d\n", i);
				d = irq_desc + (tc6393xb->irq_base + i);
				d->handle(tc6393xb->irq_base + i, d, regs);
			}
		}
	}
}


static void tc6393_mask_irq(unsigned int irq)
{
	struct tc6393xb_data *tc6393 = get_irq_chipdata(irq);

	writeb(readb(tc6393->mapbase + TC6393_SYS_IMR) | 1 << (irq - tc6393->irq_base),tc6393->mapbase + TC6393_SYS_IMR);
}

static void tc6393_unmask_irq(unsigned int irq)
{
	struct tc6393xb_data *tc6393 = get_irq_chipdata(irq);

	writeb(readb(tc6393->mapbase + TC6393_SYS_IMR) & ~( 1 << (irq - tc6393->irq_base)),tc6393->mapbase + TC6393_SYS_IMR);
}

static struct irqchip tc6393_chip = {
	.ack	= tc6393_mask_irq,
	.mask	= tc6393_mask_irq,
	.unmask	= tc6393_unmask_irq,
};


static void tc6393_setup_irq(struct tc6393xb_data *tchip)
{
	int i;
	
	/*
	 * Install handler for IRQ_TC6393_HW.
	 */
	for (i = 0; i < TC6393XB_NR_IRQS; i++) {
		int irq = tchip->irq_base + i;
		irq_desc[irq].valid    = 1;
		irq_desc[irq].probe_ok = 1;
		set_irq_chip (irq, &tc6393_chip);
		set_irq_chipdata (irq, tchip);
		set_irq_handler (irq, do_level_IRQ);
		set_irq_flags(irq, IRQF_VALID | IRQF_PROBE);
	}

	set_irq_chained_handler (tchip->irq_nr, tc6393_irq_handler);
	set_irq_type (tchip->irq_nr, IRQT_FALLING);
	set_irq_data (tchip->irq_nr, tchip);
}

void tc6393_hwinit(struct device *dev)
{
	struct tc6393xb_data *tchip = (struct tc6393xb_data *)dev_get_drvdata(dev);
	struct tc6393xb_platform_data *pdata = (struct tc6393xb_platform_data *)dev->platform_data;

	if (pdata->hw_init)
		pdata->hw_init();
		
	writew(0, tchip->mapbase + TC6393_SYS_FER);

	/* Clock setting */
	writew(pdata->sys_pll2cr,   tchip->mapbase + TC6393_SYS_PLL2CR);
	writew(pdata->sys_ccr,      tchip->mapbase + TC6393_SYS_CCR);
	writew(pdata->sys_mcr,      tchip->mapbase + TC6393_SYS_MCR);

  	/* GPIO */
	writew(pdata->sys_gper,     tchip->mapbase + TC6393_SYS_GPER);
	writew(pdata->sys_gpodsr1,  tchip->mapbase + TC6393_SYS_GPODSR1);
	writew(pdata->sys_gpooecr1, tchip->mapbase + TC6393_SYS_GPOOECR1);
}


#ifdef CONFIG_PM

static int tc6393_suspend(struct device *dev, u32 state)
{
	struct tc6393xb_platform_data *pdata = (struct tc6393xb_platform_data *)dev->platform_data;

	if (pdata->suspend)
		pdata->suspend();

	return 0;
}

static int tc6393_resume(struct device *dev)
{
	struct tc6393xb_platform_data *pdata = (struct tc6393xb_platform_data *)dev->platform_data;

	if (pdata->resume)
		pdata->resume();

	tc6393_hwinit(dev);

	return 0;
}

#else
#define tc6393_suspend NULL
#define tc6393_resume NULL
#endif

static int tc6393_probe(struct device *dev)
{

	int i;
	struct platform_device *pdev = to_platform_device (dev);
	struct tc6393xb_data *tc6393xb;
	unsigned long pbase = (unsigned long)pdev->resource[0].start;

	tc6393xb = kmalloc (sizeof (struct tc6393xb_data), GFP_KERNEL);
	if (!tc6393xb)
		return -ENOMEM;

	tc6393xb->irq_base = alloc_irq_space (TC6393XB_NR_IRQS);
	tc6393xb->irq_nr = pdev->resource[1].start;

	if (tc6393xb->irq_base == -1) {
		printk("tc6393xb: could not allocate %d irqs\n", TC6393XB_NR_IRQS);
		kfree (tc6393xb);
		return -ENOMEM;
	}

	dev_set_drvdata(dev,tc6393xb);

	tc6393xb->mapbase = (unsigned long)ioremap ((unsigned long)pdev->resource[0].start, (unsigned long)(pdev->resource[0].end - pdev->resource[0].start));

	tc6393_setup_irq (tc6393xb);

	tc6393xb->ndevices = ARRAY_SIZE (tc6393xb_blocks);
	tc6393xb->devices = kmalloc (tc6393xb->ndevices * sizeof (struct platform_device), GFP_KERNEL);
	
	for (i = 0; i < tc6393xb->ndevices; i++) {
		struct platform_device *sdev = &tc6393xb->devices[i];
		struct tc6393xb_block *blk = &tc6393xb_blocks[i];
		struct resource *res;

		/* Allocat the subdevices platform_device struct */
		memset (sdev, 0, sizeof (*sdev));
		sdev->id = blk->id.id;
		sdev->name = blk->name;
		sdev->dev.parent = dev;
		sdev->dev.platform_data = (void *)blk->hwconfig;
		sdev->num_resources = 1; // All devices have a CTL reg.
		sdev->num_resources += blk->cnf_start?1:0;
		sdev->num_resources += (blk->irq >= 0)?1:0;
//		sdev->device.release = release;

		/* Allocate space for the subdevice resources */
		res = kmalloc (sdev->num_resources * sizeof (struct resource), GFP_KERNEL);
		sdev->resource = res;
		memset (res, 0, sdev->num_resources * sizeof (struct resource));

		/* Setup the subdevice resources */
		res->start = pbase + blk->ctl_start;
		res->end   = pbase + blk->ctl_end;
		res->flags = IORESOURCE_MEM | IORESOURCE_PLATFORM_VIRTUAL;
		if(blk->cnf_start) {
			res++;
			res->start = pbase + blk->cnf_start;
			res->end   = pbase + blk->cnf_end;
			res->flags = IORESOURCE_MEM | IORESOURCE_PLATFORM_VIRTUAL;
		}
		if(blk->irq >= 0) {
			res++;
			res->start = res->end = blk->irq + tc6393xb->irq_base;
			res->flags = IORESOURCE_IRQ | IORESOURCE_PLATFORM_VIRTUAL;
		}
		platform_device_register (sdev); 
	}

	tc6393_hwinit(dev);
	
	// setup irq stuff
	writew(0, tc6393xb->mapbase + TC6393_SYS_IRR);
	writew(0xbf, tc6393xb->mapbase + TC6393_SYS_IMR);
	
	printk("Toshiba TC6393XB: Revision %d found\n", readw(tc6393xb->mapbase + TC6393_SYS_RIDR));
	
	return 0;
}

static int tc6393_remove(struct device *dev)
{
	struct tc6393xb_data *tchip = dev_get_drvdata(dev);
	int i;

	for (i = 0; i < TC6393XB_NR_IRQS; i++) {
		int irq = i + tchip->irq_base;
		irq_desc[irq].valid    = 0;
		irq_desc[irq].probe_ok = 0;
		set_irq_handler (irq, NULL);
		set_irq_chip (irq, NULL);
		set_irq_chipdata (irq, NULL);
	}
	
	set_irq_chained_handler (tchip->irq_nr, NULL);

	for (i = 0; i < tchip->ndevices; i++) {
		platform_device_unregister (&tchip->devices[i]);
		kfree (tchip->devices[i].resource);
	}

	kfree (tchip->devices);

	iounmap ((void *)tchip->mapbase);
	free_irq_space (tchip->irq_base, TC6393XB_NR_IRQS);

	kfree (tchip);

	return 0;
}


static struct device_driver tc6393xb_device_driver = {
	.name		= "tc6393xb",
	.bus		= &platform_bus_type,
	.probe		= tc6393_probe,
	.remove		= tc6393_remove,
	.suspend	= tc6393_suspend,
	.resume		= tc6393_resume,
};


static int __init tc6393_init(void)
{
	int retval = 0;
	retval = driver_register (&tc6393xb_device_driver);
	return retval;
}

static void __exit tc6393_exit(void)
{
	driver_unregister(&tc6393xb_device_driver);
}

module_init(tc6393_init);
module_exit(tc6393_exit);

MODULE_DESCRIPTION("Toshiba TC6393 core driver");
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Dirk Opfer");
