#ifndef TLV_320AIC26__H
#define TLV_320AIC26__H

/*
 *
 */

#include <sound/driver.h>
#include <sound/core.h>

enum snd_tlv320_capture_source {
	SND_TLV320_CAP_SOURCE_LINE_IN,
	SND_TLV320_CAP_SOURCE_MIC
};


struct tlv320 {
	struct semaphore sem;

	u16 regs[0x29];	/* registers cache */

	unsigned int
		powered_on:1,//+
		playback_on:1,
		playback_switch_ureq:1,
		playback_clock_on:1,
		playback_stream_opened:1,//+
		capture_on:1,
		capture_switch_ureq:1,
		capture_clock_on:1,
		capture_stream_opened:1;//+

	/* -- configuration (to fill before activation) -- */
	unsigned int
		line_in_connected:1,//+
		mic_connected:1,//+
		hp_or_line_out;
	enum snd_tlv320_capture_source	capture_source;//

	void (*power_on_chip)(int on);//+
	void (*reset_pin)(void);//+
	void (*line_out_on)(int on);//+
	void (*line_in_on)(int on);
	void (*mic_on)(int on);//+

	/* ----------------------------------------------- */

};

extern int  tlv320_activate ( struct tlv320 *tlv );
extern void tlv320_deactivate ( struct tlv320 *tlv );
extern int tlv320_set_rate ( struct tlv320 *tlv, unsigned int rate );
extern int  tlv320_open_stream ( struct tlv320 * tlv, int stream );
extern void tlv320_close_stream ( struct tlv320 *tlv, int stream );
extern int  tlv320_add_mixer_controls(struct tlv320 *tlv, 
                                                struct snd_card *acard );

/* codec control functions */
extern void tlv320_power_on(int mode);
extern void tlv320_reset(void);
extern void tlv320_line_out_on( int mode);
extern void tlv320_mic_on( int mode);


#endif
