/* 
 *
 * Copyright (C) 2005 SDG Systems, LLC
 *
 * This code is released under GPL (GNU Public License) with
 * absolutely no warranty. Please see http://www.gnu.org/ for a
 * complete discussion of the GPL.
 *
 * based on code by:
 *
 * Copyright (c) 2004 Giorgio Padrin
 * Copyright (C) 2003 Hewlett-Packard Company
 * Copyright (C) 2003 Christian Pellegrin
 * Copyright (C) 2000 Lernout & Hauspie Speech Products, N.V.
 * Copyright (c) 2002 Tomas Kasparek <tomas.kasparek@seznam.cz>
 * Copyright (c) 2002 Hewlett-Packard Company
 * Copyright (c) 2000 Nicolas Pitre <nico@cam.org>
 */

#include <linux/module.h>
#include <sound/driver.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/delay.h>
#include <linux/pm.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/sound.h>
#include <linux/soundcard.h>
#include <linux/i2c.h>
#include <linux/kmod.h>
#include <linux/dma-mapping.h>

#include <asm/semaphore.h>
#include <asm/uaccess.h>
#include <asm/hardware.h>
#include <asm/dma.h>
#include <asm/arch/pxa-regs.h>
#include <asm/arch/audio.h>
#include <asm/mach-types.h>
#include "pxa-i2s.h"

#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/driver.h>
#include <sound/control.h>
#include <sound/initval.h>
#include <sound/info.h>

#include "../i2c/ak4641.h"

#if CONFIG_MACH_H4700
#include <asm/arch/hx4700-gpio.h>
#endif
#if CONFIG_MACH_HTCUNIVERSAL
#include <asm/arch/htcuniversal-gpio.h>
#endif

#undef DEBUG
#define DEBUG 1
#ifdef DEBUG
#define DPRINTK( format, x... )  printk( format, ## x )
#else
#define DPRINTK( format, x... ) while(0)
#endif

#define chip_t pxa_ak4641_t

#define MAX_DMA_SIZE	4096
#define PXA_AK4641_BSIZE 65536

static char *id = NULL; /* ID for this card */
static int max_periods_allowed = 255; /* @@@@ check on this */

MODULE_PARM(id, "s");
MODULE_PARM_DESC(id, "ID string for PXA + AK4641 soundcard.");


struct pxa_audio_stream {
    char *id;           /* identification string */
    int  stream_id;     /* numeric identification */    
    int dma_ch;         /* DMA channel used */
    u32 dcmd;           /* parameter for DMA controller */
    volatile u32 *drcmr; /* the DMA request channel to use */

    /* used to feed the DMA of the pxa250 */
    dma_addr_t buffer;      /* buffer (physical address) */
    unsigned int buffer_size;   /* buffer size */
    
    pxa_dma_desc *dma_dring;    /* DMA descriptors ring */
    char *dma_dring_store;      /* storage space for DMA descriptors
                                        ring (alignment issue) */
    dma_addr_t dma_dring_p;     /* (physical address) */
    dma_addr_t dma_dring_store_p;   
            
    unsigned int periods;       /* periods */
    unsigned int dma_descriptors;   /* DMA descriptors in ring */ 
    unsigned int dma_size;      /* DMA transfer size */
    unsigned int dma_dpp;       /* DMA descriptors per period */
    
    volatile int dma_running;   /* DMA running? */

    snd_pcm_substream_t *stream;
};

typedef enum stream_id_t {
    PLAYBACK=0,
    CAPTURE,
    MAX_STREAMS,
} stream_id_t;

typedef struct pxa_audio_stream pxa_audio_stream_t;

typedef struct snd_card_pxa_ak4641 {
    snd_card_t *card;
    snd_pcm_t *pcm;
    struct i2c_client *ak4641;
    long samplerate;
    pxa_audio_stream_t *s[MAX_STREAMS];
    snd_info_entry_t *proc_entry;
    pxa2xx_audio_ops_t *pxa_audio;
#ifdef CONFIG_PM
    struct pm_dev *pm_dev;
    int after_suspend;
#endif
} pxa_ak4641_t;


/* TODO-CHRI: check that this is alligned on a 16 byte boundary */

static struct snd_card_pxa_ak4641 *pxa_ak4641 = NULL;

#define AUDIO_RATE_DEFAULT  44100


static unsigned int period_sizes[] = { 128, 256, 512, 1024, 2048, 4096 };
 
#define PERIOD_SIZES sizeof(period_sizes) / sizeof(period_sizes[0])
 
static snd_pcm_hw_constraint_list_t hw_constraints_period_sizes = {
    .count = PERIOD_SIZES,
    .list = period_sizes,
    .mask = 0
};


static void
pxa_ak4641_set_samplerate( pxa_ak4641_t *pxa_ak4641,
    long val, int force )
{
    struct i2c_client *ak4641 = pxa_ak4641->ak4641;
    struct ak4641_cfg cfg;

    DPRINTK("set_samplerate rate: %ld\n", val);

    if (val == pxa_ak4641->samplerate && !force)
        return;

    cfg.fs = pxa_i2s_set_samplerate( val );
    cfg.format = FMT_I2S;
    
    i2c_clients_command( ak4641->adapter, I2C_AK4641_CONFIGURE, &cfg );
    pxa_ak4641->samplerate = val;

    DPRINTK("set sample rate done\n");
}

static pxa2xx_audio_ops_t audio_ops_null = {0};

static void
pxa_ak4641_audio_init( pxa_ak4641_t *pxa_ak4641 )
{
    pxa2xx_audio_ops_t         *pxa_audio = pxa_ak4641->pxa_audio;
    
    /* h3600_audio_power(0); tnb */

    pxa_i2s_init();
    mdelay(1);

    /* Enable the audio power
    if (pxa_audio->resume)
	pxa_audio->resume( pxa_audio->priv );
 */
    /* Wait for the AK4641 to wake up */
    mdelay(1);

    /* Initialize the AK4641 internal state */
    i2c_clients_command( pxa_ak4641->ak4641->adapter, I2C_AK4641_OPEN, 0 );

    pxa_ak4641_set_samplerate( pxa_ak4641, pxa_ak4641->samplerate, 1 );

    DPRINTK("%s done\n", __FUNCTION__);
}

#ifdef CONFIG_PM

static void
pxa_ak4641_audio_shutdown( pxa_ak4641_t *pxa_ak4641 )
{
    pxa2xx_audio_ops_t         *pxa_audio = pxa_ak4641->pxa_audio;

    /* disable the audio power and all signals leading to the audio chip */
    i2c_clients_command( pxa_ak4641->ak4641->adapter, I2C_AK4641_CLOSE, 0 );

    pxa_i2s_shutdown();

    if (pxa_audio->suspend)
	pxa_audio->suspend( pxa_audio->priv );
}

#endif

static void
audio_dma_setup_descriptors_ring( pxa_audio_stream_t *s )
{
    int i, j;
    unsigned int p_size, d_size, di;
    dma_addr_t p, d_buf;
    d_size = sizeof(pxa_dma_desc);

    /* how many dma descriptors? */
    s->dma_size = MAX_DMA_SIZE;
    p_size = s->buffer_size / s->periods; /* period size */
    s->dma_dpp = (p_size - 1) / s->dma_size + 1; /* dma descriptors
                                per period */ 
    
    /* allocate storage space for descriptors ring,
       included 15 bytes extra for the alignment issue */
    s->dma_dring_store = dma_alloc_coherent( NULL,
                            d_size * s->periods * s->dma_dpp + 15,
                            &s->dma_dring_store_p, GFP_KERNEL );
            
    /* setup the descriptors ring pointer, aligned on a 16 bytes boundary */
    s->dma_dring = ((dma_addr_t) s->dma_dring_store & 0xf) ?
             (pxa_dma_desc *) (((dma_addr_t) s->dma_dring_store & ~0xf) + 16) :
             (pxa_dma_desc *) s->dma_dring_store;
    s->dma_dring_p = s->dma_dring_store_p +
              (dma_addr_t) s->dma_dring - (dma_addr_t) s->dma_dring_store;
    
    /* fill the descriptors ring */
    di = 0; /* current descriptor index */
    p = s->buffer; /* p: current period (address in buffer) */
    d_buf = s->buffer; /* d_buf: address in buffer
                         for current dma descriptor */
     /* iterate over periods */
    for (i = 0; i < s->periods; i++, p += p_size) { 
        /* iterate over dma descriptors in a period */
        for (j = 0; j < s->dma_dpp; j++, di++, d_buf += s->dma_size) {
            /* link to next descriptor */
            s->dma_dring[di].ddadr = s->dma_dring_p + (di + 1) * d_size;
            if (s->stream_id == PLAYBACK) {
                s->dma_dring[di].dsadr = d_buf;
                s->dma_dring[di].dtadr = __PREG(SADR);
                s->dma_dring[di].dcmd = DCMD_TXPCDR;
            }
            else {
                s->dma_dring[di].dsadr = __PREG(SADR);
                s->dma_dring[di].dtadr = d_buf;
                s->dma_dring[di].dcmd = DCMD_RXPCDR;
            }
            s->dma_dring[di].dcmd |=
              ((p + p_size - d_buf) >= s->dma_size) ?
              s->dma_size : p + p_size - d_buf; /* transfer length */
        }
	s->dma_dring[di].dcmd |= DCMD_ENDIRQEN; /* period irq */
    }
    s->dma_dring[di - 1].ddadr = s->dma_dring_p; /* close the ring */
}

static void audio_dma_free_descriptors_ring(pxa_audio_stream_t *s)
{
    unsigned int d_size = sizeof(pxa_dma_desc);
    if (s->dma_dring_store) {
        dma_free_coherent(NULL, d_size * s->periods * s->dma_dpp + 15,
                  s->dma_dring_store, s->dma_dring_store_p);
        s->dma_dring_store = NULL;
        s->dma_dring = NULL;
        s->dma_dring_store_p = 0;
        s->dma_dring_p = 0;
    }   
}

static void audio_dma_irq( int ch, void *dev_id, struct pt_regs *regs );

static void
audio_dma_request( pxa_audio_stream_t *s,
    void (*callback)(int , void *, struct pt_regs *) )
{
	int err;
	err = pxa_request_dma(s->id, DMA_PRIO_LOW, audio_dma_irq, s);
	if (err < 0) {
		printk("panic: cannot allocate DMA for %s\n", s->id);
		/* CHRI-TODO: handle this error condition gracefully */
	}
	s->dma_ch = err;
	
	if (s->stream_id == CAPTURE) {
	  s->drcmr = &DRCMRRXSADR;
	  *(s->drcmr) = s->dma_ch | DRCMR_MAPVLD;
	}
	else {
	  s->drcmr = &DRCMRTXSADR;
	  *(s->drcmr) = s->dma_ch | DRCMR_MAPVLD;
	}
}

static void
audio_dma_free( pxa_audio_stream_t *s )
{
    DPRINTK("%s\n", __FUNCTION__);

    pxa_free_dma( s->dma_ch );

    DPRINTK("%s done\n", __FUNCTION__);
}


static u_int
audio_get_dma_pos(pxa_audio_stream_t *s)
{
	snd_pcm_substream_t * substream = s->stream;
	snd_pcm_runtime_t *runtime = substream->runtime;
	unsigned int offset_bytes;
	unsigned int offset;
	int ch = s->dma_ch;
	u32 pos;

	if (s->stream_id == CAPTURE)
		pos = DTADR(ch);
	else
		pos = DSADR(ch);
	offset_bytes =  pos - s->buffer; 
	offset = bytes_to_frames(runtime,offset_bytes);
	return offset;
}

static void
audio_stop_dma( pxa_audio_stream_t *s )
{
    DCSR(s->dma_ch) = DCSR_STOPIRQEN;
    
    while (! (DCSR(s->dma_ch) & DCSR_STOPSTATE)) {
        if (!in_interrupt())
            schedule();
    }
    
    s->dma_running = 0;
}

static void
audio_start_dma(pxa_audio_stream_t *s, int restart)
{
    /* kick the DMA */
    DDADR(s->dma_ch) = s->dma_dring_p;
    DCSR(s->dma_ch) = DCSR_RUN; 
    s->dma_running = 1;
}

static void
audio_dma_irq(int chan, void *dev_id, struct pt_regs *regs)
{
	pxa_audio_stream_t *s = (pxa_audio_stream_t *) dev_id;
	int ch = s->dma_ch;
	u32 dcsr;

	dcsr = DCSR(ch);

	DCSR(ch) = dcsr & ~DCSR_STOPIRQEN;

	if (dcsr & DCSR_BUSERR) {
		printk("bus error\n");
		return;
	}

	if (dcsr & DCSR_ENDINTR) {
		snd_pcm_period_elapsed(s->stream);
		return;
	}
}

static int
snd_card_pxa_ak4641_pcm_trigger( stream_id_t stream_id,
    snd_pcm_substream_t * substream, int cmd )
{
    pxa_ak4641_t *chip = snd_pcm_substream_chip(substream);
#if DEBUG
    snd_pcm_runtime_t *runtime = substream->runtime;    
#endif
    
    DPRINTK("pcm_trigger id: %d cmd: %d\n", stream_id, cmd);

    DPRINTK("  sound: %d x %d [Hz]\n", runtime->channels, runtime->rate);
    DPRINTK("  periods: %ld x %ld [fr]\n", (unsigned long)runtime->periods,
        (unsigned long) runtime->period_size);
    DPRINTK("  buffer_size: %ld [fr]\n", (unsigned long)runtime->buffer_size);
    DPRINTK("  dma_addr %p\n", (char *)runtime->dma_addr);
    DPRINTK("  dma_running PLAYBACK %d CAPTURE %d\n", 
        chip->s[PLAYBACK]->dma_running, chip->s[CAPTURE]->dma_running);


    switch (cmd) {
        case SNDRV_PCM_TRIGGER_START:
        case SNDRV_PCM_TRIGGER_RESUME:
        case SNDRV_PCM_TRIGGER_PAUSE_PUSH:
            DPRINTK("trigger START\n");
            if (chip->s[PLAYBACK]->dma_running) {
                audio_stop_dma(chip->s[PLAYBACK]);
            }
            if (chip->s[CAPTURE]->dma_running) {
                audio_stop_dma(chip->s[CAPTURE]);
            }
            audio_start_dma(chip->s[stream_id],0);
            break;
        case SNDRV_PCM_TRIGGER_SUSPEND:
            /* requested stream suspend */
            audio_stop_dma(chip->s[stream_id]);
            break;
        case SNDRV_PCM_TRIGGER_STOP:
        case SNDRV_PCM_TRIGGER_PAUSE_RELEASE:
            DPRINTK("trigger STOP\n");
            audio_stop_dma(chip->s[stream_id]);
            break;
        default:
            DPRINTK("trigger UNKNOWN\n");
            return -EINVAL;
            break;
    }
    return 0;

}

static snd_pcm_hardware_t snd_pxa_ak4641_capture =
{
    .info           = (SNDRV_PCM_INFO_INTERLEAVED |
                        SNDRV_PCM_INFO_BLOCK_TRANSFER |
                        SNDRV_PCM_INFO_MMAP | SNDRV_PCM_INFO_MMAP_VALID),
    .formats        = SNDRV_PCM_FMTBIT_S16_LE,
    .rates          = (SNDRV_PCM_RATE_8000 | SNDRV_PCM_RATE_16000 |\
                        SNDRV_PCM_RATE_22050 | \
                        SNDRV_PCM_RATE_44100 | SNDRV_PCM_RATE_48000 |\
                        SNDRV_PCM_RATE_KNOT),
    .rate_min           = 8000,
    .rate_max           = 48000,
    .channels_min       = 2,
    .channels_max       = 2,
    .buffer_bytes_max   = 65536, /* was 16380 */
    .period_bytes_min   = 64, /* was 128 but this is what sa11xx-uda1341 uses */
    .period_bytes_max   = 4096, /* <= MAX_DMA_SIZE from ams/arch-sa1100/dma.h */
    .periods_min        = 2,
    .periods_max        = 32,
    .fifo_size          = 0,
};

static snd_pcm_hardware_t snd_pxa_ak4641_playback =
{
    .info           = (SNDRV_PCM_INFO_INTERLEAVED |
                        SNDRV_PCM_INFO_BLOCK_TRANSFER |
                        SNDRV_PCM_INFO_MMAP | SNDRV_PCM_INFO_MMAP_VALID |
                        SNDRV_PCM_INFO_PAUSE | SNDRV_PCM_INFO_RESUME),
    .formats        = SNDRV_PCM_FMTBIT_S16_LE,
    .rates          = (SNDRV_PCM_RATE_8000 | SNDRV_PCM_RATE_16000 |
                        SNDRV_PCM_RATE_22050 | 
                        SNDRV_PCM_RATE_44100 | SNDRV_PCM_RATE_48000 |
                        SNDRV_PCM_RATE_KNOT),
    .rate_min           = 8000,
    .rate_max           = 48000,
    .channels_min       = 2,
    .channels_max       = 2,
    .buffer_bytes_max   = 65536, 
    .period_bytes_min   = 64, /* was 128 @@@@ changed to same as sa11xx-uda1341 jamey 2003-nov-20 */
    .period_bytes_max   = 4096, /* <= MAX_DMA_SIZE from ams/arch-sa1100/dma.h */
    .periods_min        = 2,
    .periods_max        = 32, // dropped from 255 to  lower overhead costs
    .fifo_size          = 0,
};

static unsigned int rates[] = {
    8000,  
    16000, 22050,
    44100, 48000,
};

#define RATES sizeof(rates) / sizeof(rates[0])

static snd_pcm_hw_constraint_list_t hw_constraints_rates = {
    .count  = RATES,
    .list   = rates,
    .mask   = 0,
};

static int
snd_card_pxa_ak4641_playback_open( snd_pcm_substream_t * substream )
{
    pxa_ak4641_t *chip = snd_pcm_substream_chip(substream);
    snd_pcm_runtime_t *runtime = substream->runtime;
    pxa2xx_audio_ops_t         *pxa_audio = chip->pxa_audio;
    int err;
        
    DPRINTK("playback_open\n");

#ifdef CONFIG_PM
    if (chip->after_suspend) {
        chip->after_suspend = 0;
    }
#endif
    if (pxa_audio->startup)
	pxa_audio->startup( substream, pxa_audio->priv );

    chip->s[PLAYBACK]->stream = substream;

    runtime->hw = snd_pxa_ak4641_playback;
    runtime->hw.periods_max = max_periods_allowed;

/* @@@@ check this out also -jamey 2003-nov-20 */
#if 1
    snd_pcm_hw_constraint_list( runtime, 0,
                                   SNDRV_PCM_HW_PARAM_PERIOD_BYTES,
                                   &hw_constraints_period_sizes );
    snd_pcm_hw_constraint_minmax( runtime,
        SNDRV_PCM_HW_PARAM_BUFFER_BYTES,
        PXA_AK4641_BSIZE, PXA_AK4641_BSIZE );
#else
    if ((err = snd_pcm_hw_constraint_integer(runtime,
                SNDRV_PCM_HW_PARAM_PERIODS)) < 0)
        return err;
#endif

    if ((err = snd_pcm_hw_constraint_list( runtime, 0,
                SNDRV_PCM_HW_PARAM_RATE, &hw_constraints_rates )) < 0)
        return err;
        
    return 0;
}

static int
snd_card_pxa_ak4641_playback_close( snd_pcm_substream_t * substream )
{
    pxa_ak4641_t *chip = snd_pcm_substream_chip(substream);
    pxa2xx_audio_ops_t         *pxa_audio = chip->pxa_audio;

    DPRINTK("%s\n", __FUNCTION__);

    chip->s[PLAYBACK]->dma_running = 0;
    chip->s[PLAYBACK]->stream = NULL;
      
    if (pxa_audio->shutdown)
	pxa_audio->shutdown( substream, pxa_audio->priv );

    return 0;
}

static int
snd_card_pxa_ak4641_playback_ioctl( snd_pcm_substream_t * substream,
    unsigned int cmd, void *arg )
{
    DPRINTK("playback_ioctl cmd: %d\n", cmd);
    return snd_pcm_lib_ioctl(substream, cmd, arg);
}

static int
snd_card_pxa_ak4641_playback_prepare( snd_pcm_substream_t * substream )
{
    pxa_ak4641_t *chip = snd_pcm_substream_chip(substream);
    snd_pcm_runtime_t *runtime = substream->runtime;
        
    DPRINTK("playback_prepare\n");

    /* set requested samplerate */
    pxa_ak4641_set_samplerate(chip, runtime->rate, 0);
        
    return 0;
}

static int
snd_card_pxa_ak4641_playback_trigger( snd_pcm_substream_t * substream,
    int cmd )
{
    DPRINTK("playback_trigger\n");
    return snd_card_pxa_ak4641_pcm_trigger(PLAYBACK, substream, cmd);
}

static snd_pcm_uframes_t
snd_card_pxa_ak4641_playback_pointer( snd_pcm_substream_t * substream )
{
    pxa_ak4641_t *chip = snd_pcm_substream_chip(substream);
    snd_pcm_uframes_t pos;

    // DPRINTK("playback_pointer\n");        
    pos = audio_get_dma_pos(chip->s[PLAYBACK]);
    return pos;
}

static int
snd_card_pxa_ak4641_capture_open( snd_pcm_substream_t * substream )
{
    pxa_ak4641_t *chip = snd_pcm_substream_chip(substream);
    snd_pcm_runtime_t *runtime = substream->runtime;
    int err;
    
    DPRINTK("%s: \n", __FUNCTION__);

#ifdef CONFIG_PM
    if (chip->after_suspend) {
        chip->after_suspend = 0;
    }
#endif

    chip->s[CAPTURE]->stream = substream;
    
    runtime->hw = snd_pxa_ak4641_capture;
    runtime->hw.periods_max = max_periods_allowed;

/* @@@@ check this out also -jamey 2003-nov-20 */
#if 1 
    snd_pcm_hw_constraint_list( runtime, 0,
        SNDRV_PCM_HW_PARAM_PERIOD_BYTES, &hw_constraints_period_sizes );
    snd_pcm_hw_constraint_minmax( runtime, SNDRV_PCM_HW_PARAM_BUFFER_BYTES,
        PXA_AK4641_BSIZE, PXA_AK4641_BSIZE );
#else
    if ((err = snd_pcm_hw_constraint_integer( runtime,
                SNDRV_PCM_HW_PARAM_PERIODS )) < 0)
        return err;
#endif

    if ((err = snd_pcm_hw_constraint_list( runtime, 0,
                SNDRV_PCM_HW_PARAM_RATE, &hw_constraints_rates )) < 0)
        return err;
        
    return 0;
}

static int
snd_card_pxa_ak4641_capture_close( snd_pcm_substream_t * substream )
{
    pxa_ak4641_t *chip = snd_pcm_substream_chip(substream);

    DPRINTK("%s:\n", __FUNCTION__);

    /* @@@ check this -- sa11xx-uda1341 does not do this */
    chip->s[CAPTURE]->dma_running = 0;
    chip->s[CAPTURE]->stream = NULL;
      
    return 0;
}

static int
snd_card_pxa_ak4641_capture_ioctl( snd_pcm_substream_t * substream,
    unsigned int cmd, void *arg )
{
    DPRINTK("%s:\n", __FUNCTION__);
    return snd_pcm_lib_ioctl(substream, cmd, arg);
}

static int
snd_card_pxa_ak4641_capture_prepare( snd_pcm_substream_t * substream )
{
    pxa_ak4641_t *chip = snd_pcm_substream_chip(substream);
    snd_pcm_runtime_t *runtime = substream->runtime;

    DPRINTK("%s\n", __FUNCTION__);

    /* set requested samplerate */
    pxa_ak4641_set_samplerate(chip, runtime->rate, 0);
    chip->samplerate = runtime->rate;

    return 0;
}

static int
snd_card_pxa_ak4641_capture_trigger( snd_pcm_substream_t * substream,
    int cmd )
{
    DPRINTK("%s:\n", __FUNCTION__);
    return snd_card_pxa_ak4641_pcm_trigger(CAPTURE, substream, cmd);
}

static snd_pcm_uframes_t
snd_card_pxa_ak4641_capture_pointer( snd_pcm_substream_t * substream )
{
    pxa_ak4641_t *chip = snd_pcm_substream_chip(substream);
    snd_pcm_uframes_t pos;

    DPRINTK("%s:\n", __FUNCTION__);
    pos = audio_get_dma_pos(chip->s[CAPTURE]);
    return pos;
}

static int
snd_pxa_ak4641_hw_params( snd_pcm_substream_t * substream,
    snd_pcm_hw_params_t * hw_params )
{
    pxa_ak4641_t *chip = snd_pcm_substream_chip(substream);       

    int err;

    /* allocate the buffer */
    err = snd_pcm_lib_malloc_pages(substream, params_buffer_bytes(hw_params));
    if (err<0)
        printk("snd_pcm_lib_malloc_pages failed!\n");
    
    /* setup the dma descriptors ring, that refers the buffer;
       release the old one before in case, since hw_params
       may be called more than one time*/
    if (chip->s[PLAYBACK]->stream == substream) {
        if (chip->s[PLAYBACK]->dma_dring)
            audio_dma_free_descriptors_ring(chip->s[PLAYBACK]);
        chip->s[PLAYBACK]->buffer = substream->runtime->dma_addr;
        chip->s[PLAYBACK]->buffer_size = params_buffer_bytes(hw_params);
        chip->s[PLAYBACK]->periods = params_periods(hw_params);
        audio_dma_setup_descriptors_ring(chip->s[PLAYBACK]);
    }
    else {
        if (chip->s[CAPTURE]->dma_dring)
            audio_dma_free_descriptors_ring(chip->s[CAPTURE]);
        chip->s[CAPTURE]->buffer = substream->runtime->dma_addr;
        chip->s[CAPTURE]->buffer_size = params_buffer_bytes(hw_params);
        chip->s[CAPTURE]->periods = params_periods(hw_params);
        audio_dma_setup_descriptors_ring(chip->s[CAPTURE]);
    }
    
    return err;
}

static int
snd_pxa_ak4641_hw_free( snd_pcm_substream_t * substream )
{

    DPRINTK("hw_free\n");

    return snd_pcm_lib_free_pages(substream);
}

static snd_pcm_ops_t snd_card_pxa_ak4641_playback_ops = {
    .open           = snd_card_pxa_ak4641_playback_open,
    .close          = snd_card_pxa_ak4641_playback_close,
    .ioctl          = snd_card_pxa_ak4641_playback_ioctl,
    .hw_params      = snd_pxa_ak4641_hw_params,
    .hw_free        = snd_pxa_ak4641_hw_free,
    .prepare        = snd_card_pxa_ak4641_playback_prepare,
    .trigger        = snd_card_pxa_ak4641_playback_trigger,
    .pointer        = snd_card_pxa_ak4641_playback_pointer,
};

static snd_pcm_ops_t snd_card_pxa_ak4641_capture_ops = {
    .open           = snd_card_pxa_ak4641_capture_open,
    .close          = snd_card_pxa_ak4641_capture_close,
    .ioctl          = snd_card_pxa_ak4641_capture_ioctl,
    .hw_params      = snd_pxa_ak4641_hw_params,
    .hw_free        = snd_pxa_ak4641_hw_free,
    .prepare        = snd_card_pxa_ak4641_capture_prepare,
    .trigger        = snd_card_pxa_ak4641_capture_trigger,
    .pointer        = snd_card_pxa_ak4641_capture_pointer,
};


static int __init
snd_card_pxa_ak4641_pcm( pxa_ak4641_t *pxa_ak4641,
    int device, int substreams )
{
    int err;
    
    DPRINTK("%s\n", __FUNCTION__);
    // audio power is turned on and a clock is set at the same time
    // so this gives us a default starting rate
    pxa_ak4641->samplerate = AUDIO_RATE_DEFAULT;

    if ((err = snd_pcm_new( pxa_ak4641->card, "AK4641 PCM", device,
                   substreams, substreams, &(pxa_ak4641->pcm) )) < 0)
        return err;

    // this sets up our initial buffers and sets the dma_type to isa.
    // isa works but I'm not sure why (or if) it's the right choice
    // this may be too large, trying it for now

    err = snd_pcm_lib_preallocate_pages_for_all( pxa_ak4641->pcm,
            SNDRV_DMA_TYPE_DEV, NULL, 64*1024, 64*1024 );
    if (err < 0) {
        printk("preallocate failed with code %d\n", err);
    }
    DPRINTK("%s memory prellocation done\n", __FUNCTION__);
    
    snd_pcm_set_ops( pxa_ak4641->pcm, SNDRV_PCM_STREAM_PLAYBACK,
            &snd_card_pxa_ak4641_playback_ops);
    snd_pcm_set_ops( pxa_ak4641->pcm, SNDRV_PCM_STREAM_CAPTURE,
            &snd_card_pxa_ak4641_capture_ops);
    pxa_ak4641->pcm->private_data = pxa_ak4641;
    pxa_ak4641->pcm->info_flags = 0;
    strcpy( pxa_ak4641->pcm->name, "AK4641 PCM" );

    pxa_ak4641->s[PLAYBACK] = kcalloc( 1, sizeof(pxa_audio_stream_t),
                                GFP_KERNEL);
    pxa_ak4641->s[CAPTURE] = kcalloc( 1, sizeof(pxa_audio_stream_t),
                                GFP_KERNEL);

    pxa_ak4641_audio_init( pxa_ak4641 );

    /* setup naming */
    pxa_ak4641->s[PLAYBACK]->id = "AK4641 PLAYBACK";
    pxa_ak4641->s[CAPTURE]->id = "AK4641 CAPTURE";

    pxa_ak4641->s[PLAYBACK]->stream_id = PLAYBACK;
    pxa_ak4641->s[CAPTURE]->stream_id = CAPTURE;

    /* setup DMA controller */
    audio_dma_request( pxa_ak4641->s[PLAYBACK], audio_dma_irq );
    audio_dma_request( pxa_ak4641->s[CAPTURE], audio_dma_irq );

    pxa_ak4641->s[PLAYBACK]->dma_running = 0;
    pxa_ak4641->s[CAPTURE]->dma_running = 0;

    return 0;
}


/* }}} */

/* {{{ module init & exit */

#ifdef CONFIG_PM

static int
pxa_ak4641_pm_callback( struct pm_dev *pm_dev, pm_request_t req, void *data )
{
    pxa_ak4641_t *pxa_ak4641 = pm_dev->data;
    pxa_audio_stream_t *is, *os; 
    snd_card_t *card;
    
    DPRINTK("pm_callback\n");
    DPRINTK("%s: req = %d\n", __FUNCTION__, req);
    
    is = pxa_ak4641->s[PLAYBACK];
    os = pxa_ak4641->s[CAPTURE];
    card = pxa_ak4641->card;
    
    switch (req) {
        case PM_SUSPEND: /* enter D1-D3 */
            if (os)
                audio_dma_free(os);
            if (is)
                audio_dma_free(is);
            pxa_ak4641_audio_shutdown( pxa_ak4641 );
            break;
        case PM_RESUME:  /* enter D0 */
            pxa_ak4641_audio_init( pxa_ak4641 );
            if (is)
                audio_dma_request(is, audio_dma_irq);
            if (os)
                audio_dma_request(os, audio_dma_irq);
#ifdef CONFIG_PM
            pxa_ak4641->after_suspend = 1;
#endif
            if (is->dma_running) {
                audio_start_dma(is, 1);
            }
            if (os->dma_running) {
                audio_start_dma(os, 1);
            }
            break;
    }
    DPRINTK("%s: exiting...\n", __FUNCTION__);  
        return 0;
    
}

#endif

void
snd_pxa_ak4641_free( snd_card_t *card )
{
    pxa_ak4641_t *chip;
    
    chip = card->private_data;

    DPRINTK("snd_pxa_ak4641_free\n");

#ifdef CONFIG_PM
    pm_unregister(chip->pm_dev);
#endif

    chip->s[PLAYBACK]->drcmr = 0;
    chip->s[CAPTURE]->drcmr = 0;

    audio_dma_free(chip->s[PLAYBACK]);
    audio_dma_free(chip->s[CAPTURE]);

    kfree(chip->s[PLAYBACK]);
    kfree(chip->s[CAPTURE]);

    chip->s[PLAYBACK] = NULL;
    chip->s[CAPTURE] = NULL;

    kfree(chip);
    card->private_data = NULL;
}

static int __init
pxa_ak4641_init(void)
{
    int err;
    snd_card_t *card;
    struct i2c_client * ak4641_i2c_get_client( void );

    DPRINTK("pxa_ak4641_init\n");
        
    if (!machine_is_h4700())
        return -ENODEV;

    /* register the soundcard */
    card = snd_card_new(-1, id, THIS_MODULE, 0);
    if (card == NULL)
        return -ENOMEM;
    pxa_ak4641 = kcalloc( 1, sizeof *pxa_ak4641, GFP_KERNEL );
    if (pxa_ak4641 == NULL)
        return -ENOMEM;
         
    card->private_data = (void *)pxa_ak4641;
    card->private_free = snd_pxa_ak4641_free;
        
    pxa_ak4641->card = card;
#if CONFIG_MACH_H4700
    if (machine_is_h4700()) {
        extern pxa2xx_audio_ops_t hx4700_audio_ops;
        pxa_ak4641->pxa_audio = &hx4700_audio_ops;
	if (hx4700_audio_ops.resume)
	    hx4700_audio_ops.resume( hx4700_audio_ops.priv );
    }
    else
#endif
#if CONFIG_MACH_HTCUNIVERSAL
    if (machine_is_htcuniversal()) {
        extern pxa2xx_audio_ops_t htcuniversal_audio_ops;
        pxa_ak4641->pxa_audio = &htcuniversal_audio_ops;
	if (htcuniversal_audio_ops.resume)
	    htcuniversal_audio_ops.resume( htcuniversal_audio_ops.priv );
    }
    else
#endif
        pxa_ak4641->pxa_audio = &audio_ops_null;

#ifdef CONFIG_PM
    pxa_ak4641->pm_dev = pm_register( PM_SYS_DEV, 0, pxa_ak4641_pm_callback );
    if (pxa_ak4641->pm_dev)
        pxa_ak4641->pm_dev->data = pxa_ak4641;
    pxa_ak4641->after_suspend = 0;
#endif
        
    strcpy(card->driver, "AK4641");
    strcpy(card->shortname, "PXA AK4641");
    sprintf(card->longname, "Compaq iPAQ PXA with AKM AK4641");
        
    /* try to bring in i2c support */
    request_module("i2c-pxa");

    /* missing! = i2c_get_client( I2C_DRIVERID_AK4641, I2C_ALGO_PXA, NULL ); */
    pxa_ak4641->ak4641 = ak4641_i2c_get_client();

    printk("pxa_ak4641_init: card=%p chip=%p clnt=%p\n",
           pxa_ak4641->card, pxa_ak4641, pxa_ak4641->ak4641);
    if (!pxa_ak4641->ak4641) {
        printk("cannot find ak4641!\n");
    }
    else {
#if 0 /* XXX fix */
        // mixer
        if ((err = snd_card_ak4641_mixer_new( pxa_ak4641->card ))) {
            printk("snd_card_ak4641_mixer_new failed\n");
            goto nodev;
        }
#endif
        // PCM
        if ((err = snd_card_pxa_ak4641_pcm( pxa_ak4641, 0, 2 )) < 0) {
            printk("snd_card_ak4641_pcm failed\n");
            goto nodev;
        }
    }

    if ((err = snd_card_register(card)) == 0) {
        printk("iPAQ audio support initialized\n" );
        return 0;
    }

 nodev:
    snd_card_free(card);
    return err;
}

static void __exit
pxa_ak4641_exit(void)
{
    if (pxa_ak4641) {
#if 0 /* XXX fix */
        snd_card_ak4641_mixer_del( pxa_ak4641->card );
#endif
        snd_card_free( pxa_ak4641->card );
        pxa_ak4641 = NULL;
    }
}

module_init(pxa_ak4641_init);
module_exit(pxa_ak4641_exit);

MODULE_AUTHOR("Todd Blumer <todd@sdgsystems.com>");
MODULE_DESCRIPTION("PXA AK4641 sound driver");
MODULE_LICENSE( "GPL" );

