/*
 * 
 * Texas Instruments tlv320aic26 audio codec support
 * for b4 platform
 * History:
 * 2007-03-28: initial release ( Yuri Ludkevich and Evgeniy Gorlov, STC Metrotek)
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/pm.h>
#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/arch/pxa-regs.h>
#include <linux/platform_device.h>

#include "pxa2xx-i2sound.h"
#include <sound/tlv320aic26.h>  


static struct tlv320 tlv = {

  	.line_in_connected	= 0,
	.mic_connected		= 1,
	.capture_source		= SND_TLV320_CAP_SOURCE_MIC,
	.power_on_chip		= tlv320_power_on,
	.reset_pin		= tlv320_reset,
	.line_out_on		= tlv320_line_out_on,
	.mic_on			= tlv320_mic_on
};


static struct work_struct tlv320_work;
unsigned int data;

static void set_rate( void *rate ){
  tlv320_set_rate ( &tlv, *((unsigned int*) rate) );
}

static int snd_b4_audio_activate ( void ) 
{
  INIT_WORK( &tlv320_work, set_rate, (void*) &data);
  return  tlv320_activate (  &tlv );
};

static void snd_b4_audio_deactivate (void) 
{
  tlv320_deactivate ( &tlv );
};

static int snd_b4_audio_set_rate (unsigned int rate)
{
  data = rate;
  schedule_work ( &tlv320_work );
  return 0;
}

static int snd_b4_audio_open_stream ( int stream )
{
  return tlv320_open_stream ( &tlv, stream );
}


static void snd_b4_audio_close_stream (int stream) 
{
  tlv320_close_stream ( &tlv, stream );
}

static int snd_b4_audio_add_mixer_controls (struct snd_card *acard)
{
  return tlv320_add_mixer_controls( &tlv, acard );
}


static int snd_b4_audio_suspend (pm_message_t state){return 0;};
static int snd_b4_audio_resume (void){return 0;};

extern int  tlv320_activate ( struct tlv320 *tlv );
extern void tlv320_deactivate ( struct tlv320 *tlv );
extern int  tlv320_open_stream ( struct tlv320 * tlv, int stream );
extern void tlv320_close_stream ( struct tlv320 *tlv, int stream );
extern int  tlv320_add_mixer_controls(struct tlv320 *tlv,
                                                struct snd_card *acard );

static struct snd_pxa2xx_i2sound_board b4_audio = {
	.name			= "b4_audio",
	.desc			= "b4_audio Audio [codec Texas Instruments tlv320aic26]",
	.acard_id		= "b4 Audio",
	.info			= SND_PXA2xx_I2SOUND_INFO_CLOCK_FROM_PXA |
				  SND_PXA2xx_I2SOUND_INFO_CAN_CAPTURE,
	.activate		= snd_b4_audio_activate,//+
	.deactivate		= snd_b4_audio_deactivate,//+
        .set_rate               = snd_b4_audio_set_rate,
	.open_stream		= snd_b4_audio_open_stream,//+
	.close_stream		= snd_b4_audio_close_stream,//+
	.add_mixer_controls	= snd_b4_audio_add_mixer_controls,
#ifdef CONFIG_PM
	.suspend		= snd_b4_audio_suspend,
	.resume			= snd_b4_audio_resume
#endif
};

static int __init snd_b4_audio_init( void ) 
{
  return snd_pxa2xx_i2sound_card_activate ( &b4_audio );
}

static void __exit snd_b4_audio_exit( void ) 
{
  snd_pxa2xx_i2sound_card_deactivate();
}

module_init( snd_b4_audio_init );
module_exit( snd_b4_audio_exit );

MODULE_AUTHOR("Eugeniy Gorlov");
MODULE_DESCRIPTION("B4 Audio Codec");
MODULE_LICENSE("GPL");
